/*
                                 getdrive_c


      18 Dec 1991    created by Franz Herrmann
      27 Jan 1992    checked to work with C68 v2.00
      04 Feb 1992    improved debugging

      Debugging:
         cc -v -DDEBUG_THIS -TMPram2_ -oram2_q getdrive_c
         (define DEBUG if used as module)
*/


#include <qdos.h>
#include <string.h>

#ifdef DEBUG_THIS
# define DEBUG 1
# include <stdio.h>
#endif

#define MAXLEN 36


/* getdrive() takes a filename and returns a pointer to the drivename.
   Note that the function destroys intentionally any previous pointers
   returned by itself, so if maybe necessary to copy the drivename.
   The 'dev' device is not replaced by the real directory.
*/

void *getdrive (fname)
char *fname;
{
   static char drive[MAXLEN+1];
   char *p;

   if (strlen(fname) > MAXLEN)      /* filename too long */
      return NULL;

   if (!isdevice(fname,DIRDEV))     /* no drive in filename */
   {
      getcwd(drive,MAXLEN);
      if (!isdevice(drive,DIRDEV))  /* data default faulty */
         return NULL;
   }
   else
      strcpy(drive,fname);
   
   #ifdef DEBUG
   printf("getdrive: crude drive = '%s'\n",drive);
   #endif

   if (strlen(drive) < 3)           /* cannot contain drive name */
      return NULL;

   p=strchr(drive,'_');
   if (drive[0]=='n' && isdigit(drive[1]) && drive[2]=='_')
      /* We assume that the fileserver drive has not been renamed,
         as some toolkits (eg. QRD) allow, and that NULL has a
         value of 0. */
      p=strchr(p+1,'_');
   if (p==NULL) return NULL;        /* not a drive */

   *(p+1)='\0';

   return drive;
}


#ifdef DEBUG_THIS
void main(argc, argv)
int argc; char *argv[];
{
   char *drive;
   
   printf("\nfilename = '%s'\n",argv[1]);
   drive=getdrive(argv[1]);
   if (drive==NULL)
      puts("error!");
   else
      printf("drive = '%s'\n",drive);
}
#endif /* DEBUG_THIS */
