#include <string.h>
#include <ctype.h>
#include <qdos.h>
#include <assert.h>
#include <stdlib.h>

#ifndef FMAX
# define FMAX 3
#endif

#ifdef ZOO
/* #define INDICATE_FILENAME */
extern int quiet;
#endif

int getnf (char *, char *, char *, int *, int, int, int);
int isuadir (char *);

char *nextfile (int what, char *wc, int set)
{
    static char fname[FMAX+1][50];
    static char wildcard[FMAX+1][50];
    static char drive[FMAX+1][50];
    static char directory[FMAX+1][50];
    static int count[FMAX+1];
    static int nest[FMAX+1];

    struct qdirect buffer;
    char *p, q[50];
    int i, found, extra = DIRDEV, dfl;

    assert(set >= 0 && set <= FMAX);
    if (what == 0) {
        /* initialize call */
        assert (wc != NULL);
        if (nest[set] = (wc[0] == '$')) assert (wc[1] != '\0');
        count[set] = 0;
        dfl = !isdevice (wc+nest[set], &extra);

        /* get directory from wc */
        if (dfl) getcwd (directory[set], 50);
            else strcpy (directory[set], wc+nest[set]);
        found = 0;
        for (;;) {
            if (found = isuadir (directory[set])) break;
            if ((p = strrchr (directory[set], '_')) == NULL) break;
            *(p+1) = '\0';
            if (found = isuadir (directory[set])) break;
            *p = '\0';
        }
        assert (found == 1);
        if (directory[set][strlen(directory[set])-1] != '_')
            strcat (directory[set], "_");

        /* amend directory depending on wc and find wildcard */
        if (dfl) {
            /* look for relative subdirs in wc */
            strcpy (q, directory[set]);
            strcat (q, wc+nest[set]);
            while (p = strrchr (q, '_')) {
                *p = '\0';
                if (isuadir (q)) break;
            }
            i = strlen (directory[set]);
            if (p-q > i) {
                /* move relative subdirs to directory */
                strcpy (directory[set], q);
                if (directory[set][strlen(directory[set])-1] != '_')
                    strcat (directory[set], "_");
                strcpy (wildcard[set], wc + (size_t)(p-q-i+1+nest[set]));
            } else
                /* wc refers to default directory */
                strcpy (wildcard[set], wc + nest[set]);
        } else
            /* wc contained directory */
            strcpy (wildcard[set], wc + nest[set] + strlen(directory[set]));

        /* get drive from directory */
        strcpy (drive[set], directory[set]);
        if ((drive[set][0] == 'n' || drive[set][0] == 'N')
            && isdigit (drive[set][1]) && drive[set][2] == '_')
            p = strchr (drive[set] + 3, '_');
        else
            p = strchr (drive[set], '_');
        if (*p != '_') return NULL;
        *(p+1) = '\0';

        return NULL;
    }

    assert (what == 1);
    assert (sizeof(struct qdirect) == 64);

    do {
        /* get next fname as full filename */
        i = 0;
        getnf(drive[set], directory[set], (char *)&buffer, &i, count[set],
              nest[set], 0);
        if (!i) return NULL;
        ++count[set];
        strcpy (fname[set], drive[set]);    /* full = drive + header */
        strncpy (fname[set] + strlen(drive[set]), buffer.d_name,
            (size_t)buffer.d_szname);
        fname[set][buffer.d_szname + strlen(drive[set])] = '\0';

        /* get largest path q from fname for separation */
        strcpy (q, fname[set]);
        for (;;) {
            if (isuadir(q)) break;
            if ((p = strrchr (q, '_')) == NULL) break;
            *p = '\0';
        }
        if (q[strlen(q)-1] != '_' && strcmp(q,fname[set])) strcat (q, "_");

        #ifdef INDICATE_FILENAME
        if (!quiet) printf("%-40s\r", fname[set]+strlen(q));
        #endif

    } while ( !fnmatch(fname[set]+strlen(q), wildcard[set]) );

    return fname[set];
}


int getnf (char *drive, char *dirname, char *buffer, int *c, int cmax,
           int nest, int level)
{
    int bytes;
    struct qdirect *header;
    chanid_t directory;
    static char sname[37];
    size_t slen;

    directory = io_open (dirname, (INT32)DIROPEN);

    while ((*c) <= cmax) {
        again:
        bytes = io_fstrg (directory, -1L, buffer, (short)64);
        if (bytes < 64) {
            if (level==0) *c=0;
            break;
        }
        header = (struct qdirect *)buffer;
        if (!header->d_szname) goto again;

        if (header->d_type == QF_DIR_TYPE && nest) {
            slen = (size_t)(strlen(drive) + header->d_szname);
            strcpy (sname, drive);
            strncpy (sname+strlen(drive), header->d_name,
                (size_t)header->d_szname);
            *(sname + slen) = '\0';
            *c = getnf (drive, sname, buffer, c, cmax, nest, level+1);
        } else ++(*c);
    }

    io_close (directory);
    return *c;
}
