/*
                              splitfname_c

    19 Dec 1991   created by Franz Herrmann
    10 Jan 1992   variable extention length
    27 Jan 1992   checked if still runs with C68 v2.00
    04 Feb 1992   debugging improved

    Debugging:
      slb -vc libsplit_a isadir_o getdrive_o
      cc -Ldev1_ -TMPram2_ -DDEBUG_THIS -h -oram2_q -lsplit splitfname_c
      (define DEBUG if used as module)
*/

#include <qdos.h>
#include <string.h>

#ifdef DEBUG_THIS
# define DEBUG 1
# include <stdio.h>
#endif

   
int getdrive();
int isuadir();

/* splitfname() splits the given filename fname in the subdirectory
   path, basename and extention. Subdirectories must conform to FS2
   standard. The function assumes that there is enough space for the
   different parts reserved! - Code is not optimised but save.
   'extmax' specifies maximum length of extention.
*/

int splitfname (fname, subdir, name, ext, extmax)
char *fname, *subdir, *name, *ext;
short extmax;
{
   char *p, tempname[36], tempname2[36], *tmpname,
        *datad, testsub[36], drive[36];
   int subdir_there, def_drive;

   strcpy(tempname,fname);
   tmpname=&tempname[0];

   #ifdef DEBUG
   printf("\nsplitfname: supplied filename = '%s'\n",tmpname);
   #endif

   /* find drive name */
   p=getdrive(tmpname);
   if (p==NULL) return (-1);
   strcpy(drive,p);

   /* strip off drive name, if necessary */
   if (!strncmp(tmpname,drive,strlen(drive)))
   {
      tmpname += strlen(drive);
      def_drive=0;
      datad=drive;
   }
   else
   {
      def_drive=1;
      datad=getcwd(NULL,36);
   }

   #ifdef DEBUG
   printf("splitfname: drive = '%s' --> tmpname = '%s'\n",drive,tmpname);
   printf("            datad = '%s'  def_drive = %d\n",datad,def_drive);
   #endif

   /* finds subdirectory */
   subdir_there=0; p=NULL;
   strcpy(testsub,datad);
   strcpy(tempname2,tmpname);
   do
   {
      if (p!=NULL) *p='\0';
      strcpy(&testsub[strlen(datad)],tmpname);
      #ifdef DEBUG
      printf("splitfname: checking '%s' for subdir: ",testsub);
      #endif
      if (isuadir(testsub)==1)
      {
         subdir_there=1;
         #ifdef DEBUG
         printf("yes\n");
         #endif
         break;
      }
      #ifdef DEBUG
      printf("no\n");
      #endif
   }
   while((p=strrchr(tmpname,'_'))!=NULL);

   /* store subdirectory */
   if (subdir_there)
   {
      strcpy(subdir,tmpname);
      if (strlen(tempname2)==strlen(subdir))
         tmpname=(tempname2[0]='\0');
      else
         tmpname=&tempname2[strlen(subdir)+1];
   }
   else
   {
      subdir[0]='\0';
      tmpname=&tempname2[0];
   }
   
   #ifdef DEBUG
   printf("splitfname: %ssubdirectory found (%s)\n",
      subdir_there?"":"no ", subdir);
   printf("            tmpname = %s\n",tmpname);
   #endif

   /* find extention & central file name */
   p=strrchr(tmpname,'_');
   if (p==NULL || strlen(p)>extmax+1)
      ext[0]='\0';
   else
   {
      strcpy(ext,p+1);
      *p='\0';
   }
   strcpy(name,tmpname);

   #ifdef DEBUG
   printf("splitfname: basename = '%s'   extention = '%s'   extmax = %d\n",
      name, ext, extmax);
   #endif

#if 0
   /* clean a bit */
   free (datad);
#endif

   return 0;
}


#ifdef DEBUG_THIS
void main (argc, argv)
int argc; char *argv[];
{
   int n;
   char f1[36], f2[36], f3[36];
   
   n=splitfname(argv[1],f1,f2,f3,5);
   if (n==(-1))
   {
      printf("error!   _oserr = %d\n",_oserr);
      exit(_oserr);
   }
   printf("f1='%s'\n",f1);
   printf("f2='%s'\n",f2);
   printf("f3='%s'\n",f3);
}
#endif /* DEBUG_THIS */
